/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * Portions Copyrighted 2009 Jaroslav Tulach
 */

package cz.xelfi.quoridor.freemarkerdor;

import com.sun.jersey.spi.template.TemplateProcessor;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.Writer;
import java.util.Map;
import javax.script.Bindings;
import javax.script.ScriptContext;
import javax.script.ScriptEngine;
import javax.script.ScriptEngineManager;
import javax.script.ScriptException;
import javax.ws.rs.ext.Provider;
import org.openide.filesystems.FileUtil;

/**
 *
 * @author Jaroslav Tulach <jtulach@netbeans.org>
 */
@Provider
public class FreemarkerProcessor implements TemplateProcessor {
    private ScriptEngineManager manager;

    public String resolve(String name) {
        return name;
    }

    public void writeTo(String fqn, Object model, OutputStream out) throws IOException {
        synchronized (this) {
            if (manager == null) {
                manager = new ScriptEngineManager();
            }
        }
        ScriptEngine eng = manager.getEngineByName("freemarker");
        if (eng == null) {
            throw new IOException("freemarker not found!");
        }
        Bindings bind = eng.getContext().getBindings(ScriptContext.ENGINE_SCOPE);
        if (model instanceof Map) {
            bind.putAll((Map<? extends String, ? extends Object>)model);
        }
        bind.put("model", model);

        Writer w = new OutputStreamWriter(out);

        eng.getContext().setWriter(w);
        eng.getContext().setAttribute(ScriptEngine.FILENAME, fqn, ScriptContext.ENGINE_SCOPE);
        // silly workaround for
        // http://openide.netbeans.org/issues/show_bug.cgi?id=170177
        eng.getContext().setAttribute("org.openide.filesystems.FileObject", FileUtil.createData(FileUtil.getConfigRoot(), fqn), ScriptContext.ENGINE_SCOPE);

        InputStream is = FreemarkerProcessor.class.getResourceAsStream(fqn);
        if (is == null) {
            throw new IOException("Not found " + fqn); // NOI18N
        }
        InputStreamReader r = new InputStreamReader(is, "UTF-8");
        try {
            eng.eval(r);
        } catch (ScriptException ex) {
            System.err.println("dump data: " + model);
            ex.printStackTrace();
            throw new IOException(ex);
        }
    }
}
